--****************************************************************************
-- Software: BINARY FILE INTERFACE (fileIO Xtra version)
-- Version:  0.3
-- Date:     2007/06/06
-- Author:   Valentin Schmidt
-- Contact:  fluxus@freenet.de
-- URL:      http://dasdeck.de/staff/valentin/lingo/binary_file_interface/
--
-- Requirements/Dependencies: 
-- - Xtra "fileIO"
--
-- Interface for binary file operations (PHP style)
--
-- ************************************************************************

--------------------------------------
-- opens file with specified mode, returns file pointer (xtra instance)
-- tMode: r, rb, w, wb, a, ab, rw, rwb, wr, wrb
-- tType and tCreator optional, only used in mac version when new file is created
--------------------------------------
on fopen (tFile, tMode, tType, tCreator)
  fp = xtra("fileIO").new()
  
  case (tMode) of
    "r","rb": fp.openFile(tFile, 1)
    "w","wb":
      --if not fileExists(tFile) then 
      fp.createFile(tFile)
      fp.openFile(tFile, 2)
      if stringP(tType) and stringP(tCreator) then fp.setFinderInfo(tType & tCreator)
    "rw","wr","rwb","wrb": fp.openFile(tFile, 0)
    "a","ab":
      fp.openFile(tFile, 2)
      fp.setPosition(fp.getLength())
    otherwise:
      return 0
  end case
  
  if (fp.status()<>0) then fp = 0
  return fp
end

--------------------------------------
-- closes open file
--------------------------------------
on fclose (fp)
  fp.closeFile()
  fp = 0
end

--------------------------------------
-- reads n bytes from open file, returns binary string
--------------------------------------
on fread(fp, n)
  if voidP(n) then n = fp.getLength() - fp.getPosition()
  data = ""
  repeat with i = 1 to n
    c = fp.readChar()
    if fp.status()<>0 then exit repeat
    put numtochar(chartonum(c)) after data
  end repeat
  return data
end

--------------------------------------
-- writes binary string to open file
--------------------------------------
on fwrite (fp, str)
  fp.writeString(str)
end

--------------------------------------
-- returns position in open file
--------------------------------------
on ftell (fp)
  return fp.getPosition()
end

--------------------------------------
-- sets position in open file
-- whence: either integer or symbol
-- 0 or #SEEK_SET: set position to offset bytes. (default)
-- 1 or #SEEK_CUR: set position to current position plus offset.
-- 2 or #SEEK_END: set position to end of file plus offset.
--------------------------------------
on fseek (fp, pos, whence)
  	if voidP(whence) then whence = 0
  else if ilk(whence)=#symbol then
    whence = [#SEEK_SET, #SEEK_CUR, #SEEK_END].getPos(whence)
    if whence>0 then whence=whence-1
  end if
  
  case (whence) of
    0: fp.setPosition(pos)  
    1: fp.setPosition(fp.getPosition() + pos) 
    2: fp.setPosition(fp.getLength() + pos) 
  end case
  
  return (fp.status()=0)
end

--------------------------------------
-- returns size of open file
--------------------------------------
on fsize (fp)
  return fp.getLength()
end

----------------------------------------
-- global functions, no file pointer passed
----------------------------------------

----------------------------------------
-- returns file size
----------------------------------------
on file_size (tFile)
  fp = xtra("fileIO").new()
  fp.openFile(tFile, 1)
  err = fp.status()
  if err then return err
  len = fp.getLength()
  fp.closeFile()
  fp = 0
  return len
end

----------------------------------------
-- truncates file
----------------------------------------
on file_truncate (tFile, tSize)
  fp = xtra("fileIO").new()
  fp.openFile(tFile, 0)
  data = ""
  repeat with i = 1 to tSize
    c = fp.readChar()
    if fp.status()<>0 then exit repeat
    put numtochar(chartonum(c)) after data
  end repeat
  
  fn = fp.fileName()
  fp.delete()
  fp.createFile(fn)
  fp.openFile(fn, 2)
  fp.writeString(data)
  err = fp.status()
  fp.closeFile()
  fp=0
  
  return (err=0)
end

----------------------------------------
-- deletes file
----------------------------------------
on file_delete (tFile)
  fp = xtra("fileIO").new()
  fp.openFile(tFile, 0)
  err = fp.status()
  if not (err) then err = fp.delete()
  fp = 0
  return err
end

----------------------------------------
-- 
----------------------------------------
on file_exists (tFile)
  fp = xtra("fileIO").new()
  fp.openFile(tFile, 1)
  stat = fp.status()
  fp.closeFile()
  fp = 0
  if stat=-37 then
    return 0
  else
    return 1
  end if
end

----------------------------------------
-- reads whole file, returns (binary) string
----------------------------------------
on file_get_contents (tFile, tMode)
  if voidP(tMode) then tMode="rb"
  fp = xtra("fileIO").new()
  if not objectP(fp) then return -1
  fp.openFile(tFile,1)
  err = fp.status()
  if (err) then return err
  if tMode="r" then -- pure ascii data, no numtochar(0)
    ret = fp.readFile()
  else
    
    subLengthLim = 5000 -- adjust!
    chunks = []
    tmp = ""
    
    repeat while true
      c=fp.readChar()
      if fp.status()<>0 then exit repeat
      put numtochar(chartonum(c)) after tmp
      if length(tmp) > subLengthLim then
        chunks.append(tmp)
        tmp = ""
      end if
    end repeat
    
    -- assemble as large string
    	  ret = ""
    	  repeat with str in chunks
      	    put str after ret
    	  end repeat
    	  put tmp after ret
  end if
  
  fp.closeFile()
  fp = 0
  return ret
end

----------------------------------------
-- saves (binary) string as file (tType and tCreator optional, only used in mac version)
----------------------------------------
on file_put_contents (tFile, tString, tType, tCreator)
  fp = xtra("fileIO").new()
  if not objectP(fp) then return -1
  
  fp.openFile(tFile, 1)
  err = fp.status()
  if not (err) then fp.delete()
  else if (err and not (err = -37)) then return err
  
  fp.createFile(tFile)
  err = fp.status()
  if (err) then return err
  
  fp.openFile(tFile, 2)
  err = fp.status()
  if (err) then return err
  fp.writeString(tString)
  if stringP(tType) and stringP(tCreator) then fp.setFinderInfo(tType & tCreator)
  fp.closeFile()
  fp=0
end

----------------------------------------
-- display open file dialog
----------------------------------------
on display_open (title, filterMask, defaultFileName)  
  fio=xtra("fileIO").new()  
  fio.setFilterMask(filterMask)
  fn=fio.displayOpen()
  fio=0
  return string(fn)
end

----------------------------------------
-- display save file dialog
----------------------------------------
on display_save (title, filterMask, defaultFileName)  
  fio=xtra("fileIO").new()  
  fio.setFilterMask(filterMask)
  fn=fio.displaySave(title, defaultFileName)
  fio=0
  return string(fn)
end
